package mySynchMeans;

import java.util.concurrent.Semaphore;
import java.util.concurrent.TimeUnit;

public class Dormitory {
    
    private int occupiedSeats; //guarded by 'sleepingRoom'
    private final Semaphore accessGate;
    private final Object sleepingRoom=new Object(); // a lock object, for the sake of clarity

    public Dormitory(int nSeats) {
        occupiedSeats=0;
        accessGate=new Semaphore(nSeats,true);
    }
    
    public void toSleep() throws InterruptedException{
                    System.out.println("Worker "+Thread.currentThread().getName()+" asked for sleeping");
        accessGate.acquire();
        synchronized(sleepingRoom){
                    System.out.println("    Worker "+Thread.currentThread().getName()+" entered the sleeping room");
                    System.out.flush();
            occupiedSeats++;
                    System.out.println("                                   OCCUPIED = "
                        + Integer.toString(occupiedSeats));
                    System.out.flush();
            sleepingRoom.wait();
                    System.out.println("        Worker "+Thread.currentThread().getName()+" left the sleeping room");
        }
    }
    
    public boolean trySleep() throws InterruptedException{
        boolean success;
        //the following version of tryAcquire is used because it dosn't break fairness
        success = accessGate.tryAcquire(0, TimeUnit.SECONDS);
        if (success) {
            synchronized(sleepingRoom){
                    System.out.println("--->Worker "+Thread.currentThread().getName()+" entered the sleeping room via trySleep()");
                    System.out.flush();
                occupiedSeats++;
                    System.out.println("                                   OCCUPIED = "
                        + Integer.toString(occupiedSeats));
                    System.out.flush();
                sleepingRoom.wait();
                    System.out.println("        Worker "+Thread.currentThread().getName()+" left the sleeping room");
                    System.out.flush();
            }
        }
        else {
                    System.out.println("--->outch! No seat for Worker "+Thread.currentThread().getName());
                    System.out.flush();
        }
        return success;      
    }
    
    public int wakeUpAll() throws InterruptedException {
        int occ;
        synchronized(sleepingRoom){
            occ=occupiedSeats;
            accessGate.release(occupiedSeats);
            occupiedSeats=0;
                System.out.println("O-O-O-O wakeUpAll: #released="+Integer.toString(occ)
                        + " now, available permits="
                        + Integer.toString(accessGate.availablePermits()));
                System.out.println("                                   OCCUPIED = "
                        + Integer.toString(occupiedSeats));
                System.out.flush();
            sleepingRoom.notifyAll();
        }
        return occ;
    }  
}
