package mySynchMeans;

import java.util.concurrent.atomic.AtomicBoolean;
import org.junit.AfterClass;
import static org.junit.Assert.*;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;



public class DormitoryTest {

    public DormitoryTest() {
    }

    @BeforeClass
    public static void setUpClass() {
    }

    @AfterClass
    public static void tearDownClass() {
    }

    /**
     * Test of toSleep method, of class Dormitory.
     */
    @Ignore
    @Test
    public void testTrySleepFalse() throws Exception {
        System.out.println("sleep");
        Dormitory instance = new Dormitory(1);
        //with no seats inside, toSleep() must block
        
        //TODO!
        
        boolean result = instance.trySleep();

        // TODO review the generated test code and remove the default call to fail.
        //fail("The test case is a prototype.");
    }
    
    @Ignore
    @Test
    public void testOne() throws Exception{
        fail();
    }

    // tested property: 
    // a thread that invokes toSleep() on a full dormitory must block
    @Test
    public void testToSleepBlocking() throws Exception{

        int TIMEOUT = 1000; //milliseconds
        final AtomicBoolean failedInThread = new AtomicBoolean(false);

        System.out.println("TestRunner - Test toSleep() - blocking");
        final Dormitory instance = new Dormitory(1);

        Thread user = new Thread() {
            @Override
            public void run(){
                try {
                    instance.toSleep();
                    // if we get here, no blocking occurred: ERROR!
                    // this condition will be detected in the main thread
                    
                    failedInThread.set(true);   
                } catch (InterruptedException success) {
                    success.printStackTrace();
                    //thread "user" gets interrupted only if it blocked on toSleep()
                }
            }
        };
        try {
            user.start(); // user is started
            Thread.sleep(TIMEOUT); // here we wait user to block on toSleep()
            if(failedInThread.get()) {
                fail();
            }
            else {
                user.interrupt();  //we make user quit
                user.join(TIMEOUT); //we wait user to actually terminate
                assertFalse(user.isAlive());
            }
            
        } catch (Exception unexpected) {
            fail();
        }
    }
    
    // tested property:
    // if n threads call toSleep(), all must be resumed after a proper 
    // number of wakeUpAll() invocations
    @Test
    public void testAllResumed() throws Exception{

        int TIMEOUT = 1000; //milliseconds
        int seats = 5;
        int nThreads = 11;
        final AtomicBoolean failedInThread = new AtomicBoolean(false);

        System.out.println("TestRunner - Test toSleep()/wakeUpAll() - all resume");
        final Dormitory instance = new Dormitory(seats);

        Thread user = new Thread() {
            @Override
            public void run(){
                try {
                    instance.toSleep();
                    // if we get here, no blocking occurred: ERROR!
                    // this condition will be detected in the main thread
                    
                    failedInThread.set(true);   
                } catch (InterruptedException success) {
                    success.printStackTrace();
                    //thread "user" gets interrupted only if it blocked on toSleep()
                }
            }
        };
        try {
            user.start(); // user is started
            Thread.sleep(TIMEOUT); // here we wait user to block on toSleep()
            if(failedInThread.get()) {
                fail();
            }
            else {
                user.interrupt();  //we make user quit
                user.join(TIMEOUT); //we wait user to actually terminate
                assertFalse(user.isAlive());
            }
            
        } catch (Exception unexpected) {
            fail();
        }
    }
    
    
    /**
     * Test of trySleep method, of class Dormitory.
     */
    @Ignore
    @Test
    public void testTrySleep() throws Exception {
        System.out.println("trySleep");
        Dormitory instance = new Dormitory(1);
        boolean expResult = true;
        boolean result = instance.trySleep();
        assertEquals(expResult, result);
        // TODO review the generated test code and remove the default call to fail.
        //fail("The test case is a prototype.");
    }

    /**
     * Test of wakeUpAll method, of class Dormitory.
     */
    @Ignore
    @Test
    public void testWakeUpAll() throws Exception {
        System.out.println("wakeUpAll");
        Dormitory instance = new Dormitory(1);
        instance.toSleep();
        int expResult = 1;
        int result = instance.wakeUpAll();
        assertEquals(expResult, result);
        // TODO review the generated test code and remove the default call to fail.
        //fail("The test case is a prototype.");
    }

}
